﻿#region usings
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using IdeaBlade.EntityModel;

using AspAuthentication.Code;
using System;
using IdeaBlade.EntityModel.Security;

#endregion

namespace AspAuthentication {

  public partial class Page : UserControl {

    #region ctor & load

    public Page() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      CreateEntityManager(); 
      Reset();
    }

    private void CreateEntityManager() {
      WriteMessage("Creating EntityManager ...");
      _entityManager = new NorthwindIBEntities();
    }

    #endregion

    #region Button Click Handlers

    private void btnLogin_Click(object sender, RoutedEventArgs e) {
      Login(); 
    }

    private void btnFetch_Click(object sender, RoutedEventArgs e) {
      Fetch(); 
    }

    private void btnSave_Click(object sender, RoutedEventArgs e) {
      Save();
    }

    private void btnLogout_Click(object sender, RoutedEventArgs e) {
      Logout();
    }

    #endregion

    #region Login

    private void Login() {

      ILoginCredential credential = null;

      if (rbUseWindowsAuth.IsChecked.Value) {

        // Windows authentication will use user's current Windows credentials,
        // so we don't pass anything here in the Login.  
        // Be sure to set:
        //    <authentication mode="Windows" />
        // in the web.config.

      } else {

        // Here we'll do Forms style authentication.  You should set: 
        //     <authentication mode="Forms" />
        // in the web.config.
        // The credentials will be validated against info in the membership database,
        // aspnetdb.mdf, in the App_Data folder.  
        // This requires that you have SQL Express installed.  If not, you can attach
        // the DB to your SQL Server, and be sure to modify the connection strings in the
        // web.config.

        if (rbUseManager.IsChecked.Value) {
          // This user is in the membership database, with 'admin' role assigned.
          credential = new FormsAuthenticationLoginCredential("manager", "manager!", "demo", false);
        } else {
          // This user is in the membership database, with no role assigned.
          credential = new FormsAuthenticationLoginCredential("employee", "employee!", "demo", false);
        }
      }

      WriteMessage("Logging in ...");
      Authenticator.Instance.LoginAsync(credential, null, LoginCompleted, null);
    }

    private void LoginCompleted(LoginOperation args) {
      if (args.HasError) {
        string msg = args.Error.Message;
        if (rbUseWindowsAuth.IsChecked.Value) { msg += "  To use Windows Authentication, be sure to enable this in the web.config."; }
        WriteMessage(msg);
        return;
      }

      Authenticator.Instance.DefaultAuthenticationContext = args.AuthenticationContext;

      // A UserBase is returned by default if you haven't implemented a custom IEntityLoginManager
      // and/or UserBase.
      UserBase user;
      if (Authenticator.Instance.DefaultAuthenticationContext == null) {
        user = new UserBase(new UserIdentity("Guest"));
      } else {
        user = Authenticator.Instance.DefaultAuthenticationContext.Principal as UserBase;
      }

      string loginMsg = "Logged in " + user.Name;
      if (!user.IsAuthenticated) {
        loginMsg += " - Not authenticated because the wrong authentication mode is set for the credentials passed.";
      }
      WriteMessage(loginMsg);
      gridUserInfo.DataContext = user;

      // Do some SIMPLE button enabling based on user info.
      // All real authorization checking is done server side - see EntityServerEventsHandler.cs.

      // Let's say that fetches are OK for any authenticated user.
      btnFetch.IsEnabled = user.IsAuthenticated;

      // Saves are OK for admin users.  To enable roles, set the following in web.config:
      //    <roleManager enabled="true" />
      btnSave.IsEnabled = user.IsInRole("admin");

      btnLogout.IsEnabled = args.IsCompleted;
    }

    #endregion

    #region Logout

    private void Logout() {
      WriteMessage("Logging out ...");

      var op = Authenticator.Instance.LogoutAsync(Authenticator.Instance.DefaultAuthenticationContext, args => {
        WriteMessage("Logged out");
        gridUserInfo.DataContext = Authenticator.Instance.DefaultAuthenticationContext.Principal;
        Reset();
      },
        null);
    }

    #endregion

    #region Fetch

    private void Fetch() {

      // Let's just do a simple query here.  The query may be filtered on the
      // server prior to execution based on the current user's privileges.
      var query = _entityManager.Customers;

      WriteMessage("Fetching ...");

      _entityManager.ExecuteQueryAsync<Customer>(
        query,
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
          } else {
            dg.ItemsSource = args.Results;
            WriteMessage(string.Format("Retrieved {0} customers from {1}", 
              args.Results.Count(), args.CompletedSynchronously ? "cache" : "datasource"
              ));
          }
        },
        null);
    }

    #endregion

    #region Save

    private void Save() {

      if (!_entityManager.HasChanges()) {
        WriteMessage("No changes to save.");
        return;
      }

      WriteMessage("Saving ...");

      _entityManager.SaveChangesAsync(
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
          } else if (args.Cancelled) {
            WriteMessage("Save cancelled");
          } else if (args.IsCompleted) {
            WriteMessage("Changes saved");
          } 
        },
        null);
    }

    #endregion

    private void Reset() {

      btnLogin.IsEnabled = true;
      btnFetch.IsEnabled = false;
      btnLogout.IsEnabled = false;
      btnSave.IsEnabled = false;
      dg.ItemsSource = null;
      _entityManager.Clear();

      WriteMessage("Ready to login ...");
    }

    private void WriteMessage(string msg) {
      txtStatus.Text = msg;
    }

    private NorthwindIBEntities _entityManager;
  }
}
