﻿using System;
using System.Linq;
using IdeaBlade.EntityModel;
using IdeaBlade.Core.DomainServices;

using AspAuthentication.Code;
using IdeaBlade.EntityModel.Server;
using IdeaBlade.Core;

namespace AspAuthentication {

  /// <summary>
  /// Custom implementations of EntityServerQueryInterceptor and EntityServerSaveInterceptor.
  /// Methods in these classes will be called by DevForce (on the server) during any fetch or save.
  /// </summary>
  public class EntityServerQueryManager : EntityServerQueryInterceptor {
    
    protected override bool FilterQuery() {

      // Filter the query so that the user will see only US customers unless an Admin.
      if (!Principal.IsInRole("admin")) {
        QueryFilters.AddFilter<Customer>(q => q.Where(c => c.Country == "USA"));
        return true;
      }

      return base.FilterQuery();
    }
  }

  public class EntityServerSaveManager : EntityServerSaveInterceptor {

    // You can instead apply the ClientCanSave(roles) attribute on the type.  To keep it
    // simple, we do a simple role test here.  Only admins are authorized for saves.
    protected override bool ClientCanSave(Type type) {
      return Principal.IsInRole("admin");
    }

  }

}