﻿#Region "usings"
Imports System.Collections
Imports System.Windows
Imports System.Windows.Controls
Imports IdeaBlade.EntityModel

Imports AspAuthentication.Code
Imports IdeaBlade.EntityModel.Security

#End Region

Namespace AspAuthentication

    Partial Public Class Page
        Inherits UserControl

#Region "ctor & load"

        Public Sub New()
            InitializeComponent()
            AddHandler Loaded, AddressOf Page_Loaded
        End Sub

        Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            CreateEntityManager()
            Reset()
        End Sub

        Private Sub CreateEntityManager()
            WriteMessage("Creating EntityManager ...")
            _entityManager = New NorthwindIBEntities()
        End Sub

#End Region

#Region "Button Click Handlers"

        Private Sub btnLogin_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Login()
        End Sub

        Private Sub btnFetch_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Fetch()
        End Sub

        Private Sub btnSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Save()
        End Sub

        Private Sub btnLogout_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Logout()
        End Sub

#End Region

#Region "Login"

        Private Sub Login()

            Dim credential As ILoginCredential = Nothing

            If rbUseWindowsAuth.IsChecked.Value Then

                ' Windows authentication will use user's current Windows credentials,
                ' so we don't pass anything here in the Login.  
                ' Be sure to set:
                '    <authentication mode="Windows" />
                ' in the web.config.

            Else

                ' Here we'll do Forms style authentication.  You should set: 
                '     <authentication mode="Forms" />
                ' in the web.config.
                ' The credentials will be validated against info in the membership database,
                ' aspnetdb.mdf, in the App_Data folder.  
                ' This requires that you have SQL Express installed.  If not, you can attach
                ' the DB to your SQL Server, and be sure to modify the connection strings in the
                ' web.config.

                If rbUseManager.IsChecked.Value Then
                    ' This user is in the membership database, with 'admin' role assigned.
                    credential = New FormsAuthenticationLoginCredential("manager", "manager!", "demo", False)
                Else
                    ' This user is in the membership database, with no role assigned.
                    credential = New FormsAuthenticationLoginCredential("employee", "employee!", "demo", False)
                End If
            End If

            WriteMessage("Logging in ...")
            Authenticator.Instance.LoginAsync(credential, Nothing, AddressOf LoginCompleted, Nothing)
        End Sub

        Private Sub LoginCompleted(ByVal args As LoginOperation)
            If args.HasError Then
                Dim msg As String = args.Error.Message
                If rbUseWindowsAuth.IsChecked.Value Then
                    msg &= "  To use Windows Authentication, be sure to enable this in the web.config."
                End If
                WriteMessage(msg)
                Return
            End If

            Authenticator.Instance.DefaultAuthenticationContext = args.AuthenticationContext

            ' A UserBase is returned by default if you haven't implemented a custom IEntityLoginManager
            ' and/or UserBase.
            Dim user As UserBase

            If Authenticator.Instance.DefaultAuthenticationContext Is Nothing Then
                user = New UserBase(New UserIdentity("Guest"))
            Else
                user = TryCast(Authenticator.Instance.DefaultAuthenticationContext.Principal, UserBase)
            End If


            Dim loginMsg As String = "Logged in " & user.Name
            If Not user.IsAuthenticated Then
                loginMsg &= " - Not authenticated because the wrong authentication mode is set for the credentials passed."
            End If
            WriteMessage(loginMsg)
            gridUserInfo.DataContext = user

            ' Do some SIMPLE button enabling based on user info.
            ' All real authorization checking is done server side - see EntityServerEventsHandler.cs.

            ' Let's say that fetches are OK for any authenticated user.
            btnFetch.IsEnabled = user.IsAuthenticated

            ' Saves are OK for admin users.  To enable roles, set the following in web.config:
            '    <roleManager enabled="true" />
            btnSave.IsEnabled = user.IsInRole("admin")

            btnLogout.IsEnabled = args.IsCompleted
        End Sub

#End Region

#Region "Logout"

        Private Sub Logout()
            WriteMessage("Logging out ...")

            Authenticator.Instance.LogoutAsync(Authenticator.Instance.DefaultAuthenticationContext, Sub(args)
                                                                                                        WriteMessage("Logged out")
                                                                                                        gridUserInfo.DataContext = Authenticator.Instance.DefaultAuthenticationContext.Principal
                                                                                                        Reset()
                                                                                                    End Sub, Nothing)
        End Sub

#End Region

#Region "Fetch"

        Private Sub Fetch()

            ' Let's just do a simple query here.  The query may be filtered on the
            ' server prior to execution based on the current user's privileges.
            Dim query = _entityManager.Customers

            WriteMessage("Fetching ...")

            _entityManager.ExecuteQueryAsync(Of Customer)(query, Sub(args)
                                                                     If args.HasError Then
                                                                         WriteMessage(args.Error.Message)
                                                                     Else
                                                                         dg.ItemsSource = args.Results
                                                                         WriteMessage(String.Format("Retrieved {0} customers from {1}", args.Results.Count(), If(args.CompletedSynchronously, "cache", "datasource")))
                                                                     End If
                                                                 End Sub, Nothing)
        End Sub

#End Region

#Region "Save"

        Private Sub Save()

            If Not _entityManager.HasChanges() Then
                WriteMessage("No changes to save.")
                Return
            End If

            WriteMessage("Saving ...")

            _entityManager.SaveChangesAsync(Sub(args)
                                                If args.HasError Then
                                                    WriteMessage(args.Error.Message)
                                                ElseIf args.Cancelled Then
                                                    WriteMessage("Save cancelled")
                                                ElseIf args.IsCompleted Then
                                                    WriteMessage("Changes saved")
                                                End If
                                            End Sub, Nothing)
        End Sub

#End Region

        Private Sub Reset()

            btnLogin.IsEnabled = True
            btnFetch.IsEnabled = False
            btnLogout.IsEnabled = False
            btnSave.IsEnabled = False
            dg.ItemsSource = Nothing
            _entityManager.Clear()

            WriteMessage("Ready to login ...")
        End Sub

        Private Sub WriteMessage(ByVal msg As String)
            txtStatus.Text = msg
        End Sub

        Private _entityManager As NorthwindIBEntities
    End Class
End Namespace
